subroutine linear( nmin, nmax, dr, a, deltaa, a6, al, flat )

use sweepsize

IMPLICIT NONE

! LOCALS
INTEGER :: n, nmin, nmax
REAL :: onemfl, twothd
REAL, DIMENSION(maxsweep) :: a, deltaa, a6, al, ar, da, dr, diffa, diffa2, flat
REAL, DIMENSION(maxsweep) :: datemp1, datemp2

twothd  = 2.0/3.0
a6      = 0.0  ! quadratic term set to zero for linear interpolation
datemp1 = 0.0

!----------------------------------------------------------------------

! Colella's MUSCL (1985)

do n = nmin-2, nmax+1
  diffa(n)  = a(n+1) - a(n)
enddo

! Eq. 2.5

do n = nmin-1, nmax+1
  if (diffa(n)*diffa(n-1) > 0.0) datemp1(n) = 2.0*min( abs(diffa(n)), abs(diffa(n-1)) )
  diffa2(n)  = a(n+1) - a(n-1)
  datemp2(n) = 0.25 * sign( min( 0.5*abs(diffa2(n)), datemp1(n) ), diffa2(n))
enddo


do n = nmin, nmax
  
  ! Eq. 2.6
  da(n) = sign( min( twothd*abs( diffa2(n) - datemp2(n+1) - datemp2(n-1) ), datemp1(n) ), diffa2(n))

  ! Eq. 2.7 - for nonuniform grid spacing
  da(n) = sign( min( abs(da(n)/dr(n)), 2.0*abs(diffa(n)), 2.0*abs(diffa(n-1)) ), diffa2(n))

  al(n) = a(n) - 0.5*da(n)
  ar(n) = a(n) + 0.5*da(n)

enddo

! Flattening - using the same routine as PPMLR

do n = nmin, nmax
  onemfl    = 1.0 - flat(n)
  ar(n)     = flat(n) * a(n) + onemfl * ar(n)
  al(n)     = flat(n) * a(n) + onemfl * al(n)
  deltaa(n) = ar(n) - al(n)
enddo

return
end

!#########################################################################
subroutine linset( nmin, nmax, dr, dx, xa )

use sweepsize

IMPLICIT NONE

! LOCALS
INTEGER :: n, nmin, nmax
REAL :: twothd
REAL, DIMENSION(maxsweep) :: dx, xa, dr, diffr, drtemp1, drtemp2

twothd = 2.0/3.0
drtemp1 = 0.0

!------------------------------------------------------------------------------

do n = nmin-1, nmax+2
  diffr(n) = xa(n+1) - xa(n-1)
enddo

do n = nmin-1, nmax+1
  if (diffr(n+1)*diffr(n) > 0.0) drtemp1(n) = min( abs(diffr(n+1)), abs(diffr(n)) )
  drtemp2(n) = 0.25 * sign( min( 0.5*abs(diffr(n)), drtemp1(n) ), diffr(n))
enddo

do n = nmin, nmax
   dr(n) = sign( min( twothd*abs( diffr(n) - drtemp2(n+1) - drtemp2(n-1) ), drtemp1(n) ), diffr(n) ) / dx(n)
enddo


return
end

