program mergevh1data

! read in MVH-1 data in netCDF format, sliced in j; write out concatenated netCDF datasets
! compile with:
      
IMPLICIT NONE


CHARACTER(LEN=4) :: tmp, tmp1, tmp2
CHARACTER(LEN=50) :: base
CHARACTER(LEN=50) :: filename
CHARACTER(LEN=50) :: masterfile
CHARACTER(LEN=16) :: coord1, coord2, coord3
CHARACTER(LEN=16), DIMENSION(9) :: varname, vector
CHARACTER(LEN=80) :: label1, label2, label3, label4, geometry, periodic

INTEGER, DIMENSION(9) :: varID
INTEGER :: xDimID, yDimID, zDimID, x_varID, y_varID, z_varID
INTEGER :: status, ncid, ncid_slab, ncstat

INTEGER, DIMENSION(3) :: dims, start, mdims
INTEGER :: imax, jmax, kmax, ks, nv, nvar, ipart, jjmax, kkmax

INTEGER :: n, i, j, k, rank, npe, nstart, nstop, nfile, nsteps, error, existing, npey, ngeom

REAL :: time
REAL, DIMENSION(:,:,:), ALLOCATABLE :: var, zro, zpr, zux, zuy, zuz


!######################################################################################################
open(27, file="postprocess", status="OLD", iostat=existing)
if (existing > 0) then
  write(*,*) 'Input name of file to paste'
  read(5,*) base
  write(*,*) 'Input first file number'
  read(5,*) nstart
  write(*,*) 'Input number of time steps to merge'
  read(5,*) nsteps
  write(*,*) 'Input number of mpi tasks used in simulation'
  read(5,*) npe
else
  read (27,*) base
  read (27,*) nstart
  read (27,*) nsteps
  read (27,*) npe
  close(27)
endif


!  Open up geometry file and get dimensions
filename = trim(base) // '.geo'
open(unit=15,file=filename,form='unformatted')
  read(15) label1
  read(15) geometry
  read(15) periodic
  read(15) label1
  read(15) label1
  read(15) label1
  read(15) ipart
  read(15) label1
  read(15) label1
  read(15) imax, jjmax, kkmax 
close(15)

if (periodic(1:1) == 'y') then
  jmax = jjmax - 1
else
  jmax = jjmax
endif

if (periodic(1:1) == 'z') then
  kmax = kkmax - 1
else
  kmax = kkmax
endif

print *, 'processing ', trim(base), ' files'
print *, 'grid dimensions: ', imax, jmax, kmax
print *, 'Periodic = ', periodic(1:1)
print *, 'Geometry = ', geometry(1:10)

ks = kmax / npe

nstart = nstart + 1000
nstop = nstart + nsteps - 1

! allocate data array
ALLOCATE( var(imax,jmax,ks) )

! allocate output data arrays
ALLOCATE( zro(imax,jjmax,kkmax) )
ALLOCATE( zpr(imax,jjmax,kkmax) )
ALLOCATE( zux(imax,jjmax,kkmax) )
ALLOCATE( zuy(imax,jjmax,kkmax) )
ALLOCATE( zuz(imax,jjmax,kkmax) )

!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

do nfile = nstart, nstop

 n = 0
 write(tmp1,"(i4)") nfile

 !#############################  LOOP OVER SLABS ###########################
 do n = 0, npe-1

   write(tmp2,"(i4)") n
   do i = 1, 4
     if ((tmp2(i:i)) .eq. ' ') tmp2(i:i) = '0'
   enddo
   filename   = trim(base) // '_' // tmp1 // '.' // tmp2 // '.bin'
   open(unit=15,file=filename,form='unformatted')

    read(15) var
    do k = 1, ks
     do j = 1, jmax
      do i = 1, imax
        zro(i,j,k+n*ks) = var(i,j,k)
      enddo
     enddo
    enddo
    if ((n==0).and.(periodic(1:1)=='z')) then
     do j = 1, jmax
      do i = 1, imax
        zro(i,j,kkmax) = var(i,j,1)
      enddo
     enddo
    endif
         
    read(15) var
    do k = 1, ks
     do j = 1, jmax
      do i = 1, imax
        zpr(i,j,k+n*ks) = var(i,j,k)
      enddo
     enddo
    enddo
    if ((n==0).and.(periodic(1:1)=='z')) then
     do j = 1, jmax
      do i = 1, imax
        zpr(i,j,kkmax) = var(i,j,1)
      enddo
     enddo
    endif
         
    read(15) var
    do k = 1, ks
     do j = 1, jmax
      do i = 1, imax
        zux(i,j,k+n*ks) = var(i,j,k)
      enddo
     enddo
    enddo
    if ((n==0).and.(periodic(1:1)=='z')) then
     do j = 1, jmax
      do i = 1, imax
        zux(i,j,kkmax) = var(i,j,1)
      enddo
     enddo
    endif
         
    read(15) var
    do k = 1, ks
     do j = 1, jmax
      do i = 1, imax
        zuy(i,j,k+n*ks) = var(i,j,k)
      enddo
     enddo
    enddo
    if ((n==0).and.(periodic(1:1)=='z')) then
     do j = 1, jmax
      do i = 1, imax
        zuy(i,j,kkmax) = var(i,j,1)
      enddo
     enddo
    endif
         
    read(15) var
    do k = 1, ks
     do j = 1, jmax
      do i = 1, imax
        zuz(i,j,k+n*ks) = var(i,j,k)
      enddo
     enddo
    enddo
    if ((n==0).and.(periodic(1:1)=='z')) then
     do j = 1, jmax
      do i = 1, imax
        zuz(i,j,kkmax) = var(i,j,1)
      enddo
     enddo
    endif
         
 enddo

 if (periodic(1:1) == 'y') then

   do k = 1, kmax
    do i = 1, imax
     zro(i,jjmax,k) = zro(i,1,k)
     zpr(i,jjmax,k) = zpr(i,1,k)
     zux(i,jjmax,k) = zux(i,1,k)
     zuy(i,jjmax,k) = zuy(i,1,k)
     zuz(i,jjmax,k) = zuz(i,1,k)
    enddo
   enddo

 endif

 !############################################################################
 ipart  = 1
 label2 = 'part'
 label3 = 'block'

  label4 = 'density'
  filename = trim(base) // '_' // tmp1 // '.den'
  open(unit=15,file=filename,form='unformatted')
    write(15) label4
    write(15) label2
    write(15) ipart
    write(15) label3
    write(15) zro
  close(15)
 
  label4 = 'pressure'
  filename = trim(base) // '_' // tmp1 // '.pre'
  open(unit=15,file=filename,form='unformatted')
    write(15) label4
    write(15) label2
    write(15) ipart
    write(15) label3
    write(15) zpr
  close(15)
 
  label4 = 'velocity'
  filename = trim(base) // '_' // tmp1 // '.vel'
  open(unit=15,file=filename,form='unformatted')
    write(15) label4
    write(15) label2
    write(15) ipart
    write(15) label3
    write(15) zux
    write(15) zuy
    write(15) zuz
  close(15)
 
 
enddo
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


END PROGRAM mergevh1data
