subroutine prin

! For 1D, write out separate ascii file for each call to prin
! For 2D/3D, write out binary EnSight Gold data files (casefile, geometry, variables)
!---------------------------------------------------------------------------
use global
use zone

IMPLICIT NONE

! LOCALS
logical :: periodic_in_y, periodic_in_z
character(LEN=4) :: tmp4
character(LEN=50):: filename, casefile, geomfile
character(LEN=80) :: label1a, label1b, label1c, label1d, label1e, label1f, label1g, label1h
character(LEN=80) :: label2, label3, label4 

integer :: ipart, nsteps, jjmax, kkmax
real(kind=4) :: sx(imax), sy(jmax), sz(kmax)
real(kind=4), dimension(:,:,:), allocatable :: var

!------------------------------------------------------------------------------

if (ndim == 1) then         ! Keep 1D output simple, just write out in ascii...

  ! Create filename from integer nfile (in global.mod) and prefix such that filename
  ! looks like prefx1000.dat where 1000 is the value of nfile
  write(tmp4,"(i4)") nfile + 1000
  filename = 'output/' // trim(prefix) // tmp4 // '.dat'

  open(unit=3,file=filename,form='formatted')
   do i = 1, imax
     write(3, 1003) zxa(i), zro(i,1,1),zpr(i,1,1), zux(i,1,1)
   enddo
  close(3)
  1003 format(1pe13.5,4e13.5)

  write(8,6001) trim(prefix)//tmp4//'.dat', time, ncycle

!#################################################################################

else    ! write EnSight Gold data files

! geometry options are   0,0,0; 1,3,0; 2,4,5

 kkmax = kmax
 jjmax = jmax

 periodic_in_z = .false.
 if (ngeomz == 5) then
   if ((zzc(kmax) - zzc(1)) > 6.0) then
     kkmax = kmax + 1
     periodic_in_z = .true.
   endif
 endif

 periodic_in_y = .false.
 if (ngeomy > 2) then
   if ((zyc(jmax) - zyc(1)) > 6.0) then
     jjmax = jmax + 1
     periodic_in_y = .true.
   endif

 endif

 ALLOCATE(var(imax,jjmax,kkmax) )  ! used both for data arrays and for coordinates in curvilinear geometry


! set up labels for EnSight files
label1a = 'Fortran Binary'
label1b = 'cartesian geometry file from VH-1'
label1c = 'junk line 2'
label1d = 'node id off'
label1e = 'element id off'
label1f = 'part '
label1g = 'cartesian coordinates'
label1h = 'block rectilinear'
if (ngeomy > 2) label1h = 'block curvilinear'
ipart  = 1
label2 = 'part'
label3 = 'block'

! Create filename from integer nfile (in global.h) and prefix such that filename
! looks like prefx1000.den where 1000 is the value of nfile

write(tmp4,"(i4)") nfile+1000
casefile = 'output/' // trim(prefix) // '.case'

if (nfile == 0) then   ! open up case file and fill in initial data

  open(unit=14,file=casefile)
  write(14,*) 'FORMAT'
  write(14,*) 'type: ensight gold'
  write(14,*) 'GEOMETRY'
  write(14,*) 'model: ', trim(prefix)//'.geo' !, '     change_coords_only'
  write(14,*) 'VARIABLE'
  write(14,*) 'scalar per node:    density   ', trim(prefix) // '_****.den'
  write(14,*) 'scalar per node:    pressure  ', trim(prefix) // '_****.pre'
  write(14,*) 'vector per node:    velocity  ', trim(prefix) // '_****.vel'
  write(14,*) 'TIME'
  write(14,*) 'time set:    1'
  write(14,*) 'number of steps:    ', n_pframes   ! number of data files calculated in vhone.f90
  write(14,*) 'filename start number:  ', nfile+1000
  write(14,*) 'filename increment:    1'
  write(14,fmt="('time values: ')",advance="no") 

  geomfile = 'output/' // trim(prefix) // '.geo'
  open(unit=15,file=geomfile,form='unformatted')
  write(15) label1a
  write(15) label1b
  write(15) label1c
  write(15) label1d
  write(15) label1e
  write(15) label1f
  write(15) ipart
  write(15) label1g
  write(15) label1h
  write(15) imax, jjmax, kkmax

  if (ngeomy < 3) then

    sx = zxc
    sy = zyc
    sz = zzc
    write(15) sx
    write(15) sy
    write(15) sz

  else if (ngeomy == 3) then

    do k = 1, kmax
     do j = 1, jmax
      do i = 1, imax
       var(i,j,k) = zxc(i)*cos(zyc(j))
      enddo
     enddo
    enddo

    if (periodic_in_y) then
     do k = 1, kmax
      do i = 1, imax
        var(i,jjmax,k) = var(i,1,k)
      enddo
     enddo
    endif

    write(15) var

    do k = 1, kmax
     do j = 1, jmax
      do i = 1, imax
       var(i,j,k) = zxc(i)*sin(zyc(j))
      enddo
     enddo
    enddo

    if (periodic_in_y) then
     do k = 1, kmax
      do i = 1, imax
        var(i,jjmax,k) = var(i,1,k)
      enddo
     enddo
    endif

    write(15) var

    do k = 1, kmax
     do j = 1, jmax
      do i = 1, imax
       var(i,j,k) = zzc(k)
      enddo
     enddo
    enddo

    if (periodic_in_y) then
     do k = 1, kmax
      do i = 1, imax
        var(i,jjmax,k) = var(i,1,k)
      enddo
     enddo
    endif

    write(15) var

  else

    do k = 1, kmax
     do j = 1, jmax
      do i = 1, imax
       var(i,j,k) = zxc(i)*sin(zyc(j))*cos(zzc(k))
      enddo
     enddo
    enddo

    if (periodic_in_z) then
     do j = 1, jmax
      do i = 1, imax
        var(i,j,kkmax) = var(i,j,1)
      enddo
     enddo
    endif

    write(15) var

    do k = 1, kmax
     do j = 1, jmax
      do i = 1, imax
       var(i,j,k) = zxc(i)*sin(zyc(j))*sin(zzc(k))
      enddo
     enddo
    enddo

    if (periodic_in_z) then
     do j = 1, jmax
      do i = 1, imax
        var(i,j,kkmax) = var(i,j,1)
      enddo
     enddo
    endif

    write(15) var

    do k = 1, kmax
     do j = 1, jmax
      do i = 1, imax
       var(i,j,k) = zxc(i)*cos(zyc(j))
      enddo
     enddo
    enddo

    if (periodic_in_z) then
     do j = 1, jmax
      do i = 1, imax
        var(i,j,kkmax) = var(i,j,1)
      enddo
     enddo
    endif

    write(15) var

  endif
  close(15)

else
  open(unit=14,file=casefile,form='formatted',position='append')
endif

write(14,fmt="(f9.5)",advance="no") time
if ((nfile - 10*(nfile/10)) .eq. 0) write(14,*)
close(14)

!############### DENSITY ###########################

label4 = 'density'
filename = 'output/' // trim(prefix) // '_' // tmp4 // '.den'
open(unit=15,file=filename,form='unformatted')
 write(15) label4
 write(15) label2
 write(15) ipart
 write(15) label3

 do k = 1, kmax
  do j = 1, jmax
   do i = 1, imax
     var(i,j,k) = zro(i,j,k)
   enddo
  enddo
 enddo

 if (periodic_in_y) then
  do k = 1, kmax
   do i = 1, imax
     var(i,jjmax,k) = var(i,1,k)
   enddo
  enddo
 else if (periodic_in_z) then
  do j = 1, jmax
   do i = 1, imax
     var(i,j,kkmax) = var(i,j,1)
   enddo
  enddo
 endif

 write(15) var
close(15)

!############### PRESSURE ###########################

label4 = 'pressure'
filename = 'output/' // trim(prefix) // '_' // tmp4 // '.pre'
open(unit=15,file=filename,form='unformatted')
 write(15) label4
 write(15) label2
 write(15) ipart
 write(15) label3

 do k = 1, kmax
  do j = 1, jmax
   do i = 1, imax
     var(i,j,k) = zpr(i,j,k)
   enddo
  enddo
 enddo

 if (periodic_in_y) then
  do k = 1, kmax
   do i = 1, imax
     var(i,jjmax,k) = var(i,1,k)
   enddo
  enddo
 else if (periodic_in_z) then
  do j = 1, jmax
   do i = 1, imax
     var(i,j,kkmax) = var(i,j,1)
   enddo
  enddo
 endif

 write(15) var
close(15)

!############### VELOCITY ###########################


label4 = 'velocity'
filename = 'output/' // trim(prefix) // '_' // tmp4 // '.vel'
open(unit=15,file=filename,form='unformatted')
 write(15) label4
 write(15) label2
 write(15) ipart
 write(15) label3

 do k = 1, kmax
  do j = 1, jmax
   do i = 1, imax
     var(i,j,k) = zux(i,j,k)
   enddo
  enddo
 enddo

 if (periodic_in_y) then
  do k = 1, kmax
   do i = 1, imax
     var(i,jjmax,k) = var(i,1,k)
   enddo
  enddo
 else if (periodic_in_z) then
  do j = 1, jmax
   do i = 1, imax
     var(i,j,kkmax) = var(i,j,1)
   enddo
  enddo
 endif

  write(15) var

 do k = 1, kmax
  do j = 1, jmax
   do i = 1, imax
     var(i,j,k) = zuy(i,j,k)
   enddo
  enddo
 enddo

 if (periodic_in_y) then
  do k = 1, kmax
   do i = 1, imax
     var(i,jjmax,k) = var(i,1,k)
   enddo
  enddo
 else if (periodic_in_z) then
  do j = 1, jmax
   do i = 1, imax
     var(i,j,kkmax) = var(i,j,1)
   enddo
  enddo
 endif

  write(15) var

 do k = 1, kmax
  do j = 1, jmax
   do i = 1, imax
     var(i,j,k) = zuz(i,j,k)
   enddo
  enddo
 enddo

 if (periodic_in_y) then
  do k = 1, kmax
   do i = 1, imax
     var(i,jjmax,k) = var(i,1,k)
   enddo
  enddo
 else if (periodic_in_z) then
  do j = 1, jmax
   do i = 1, imax
     var(i,j,kkmax) = var(i,j,1)
   enddo
  enddo
 endif

  write(15) var
close(15)


write(8,6000) filename, time, ncycle


endif

nfile = nfile + 1
DEALLOCATE( var )

 6000 format('Wrote time slice to ',a20,' at time =',1pe12.5,' (ncycle =', i7,')')
 6001 format('Wrote ',a16,' to disk at time =',1pe12.5,' (ncycle =', i7,')')

return
end

