subroutine prin

! For 1D, write out separate ascii file for each call to prin
! For 2D/3D, write out one silo data file containing all variables
!---------------------------------------------------------------------------
use global
use zone

IMPLICIT NONE

include 'silo.inc'

! LOCALS
character(LEN=4) :: tmp1
character(LEN=8) :: coord
character(LEN=50):: filename, filename1d
 
integer :: DBMMFile, DBFile, DB_SILO_BACKEND = DB_PDB   !DB_SILO_BACKEND = DB_HDF5
integer :: error, optionListID, ipe, oldLength, ivariable, nprefix
integer, dimension(3) :: dbdims

real, dimension(imax+1) :: zex
real, dimension(jmax+1) :: zey
real, dimension(kmax+1) :: zez

!------------------------------------------------------------------------------

if (ndim == 1) then         ! Keep 1D output simple, just write out in ascii...

  ! Create filename from integer nfile (in global.mod) and prefix such that filename
  ! looks like prefx1000.dat where 1000 is the value of nfile
  write(tmp1,"(i4)") nfile + 1000
  filename1d = 'output/' // trim(prefix) // tmp1 // '.dat'

  open(unit=3,file=filename1d,form='formatted')
   do i = 1, imax
     write(3, 1003) zxa(i), zro(i,1,1),zpr(i,1,1), zux(i,1,1)
   enddo
  close(3)
  1003 format(1pe13.5,4e13.5)

  write(8,6001) trim(prefix)//tmp1//'.dat', time, ncycle

!#################################################################################

else    ! use silo calls to write out 2D or 3D arrays

!--- Construct arrays containing zone edges:
 zex(1:imax) = zxa(1:imax) ; zex(imax+1) = zxa(imax) + zdx(imax)
 zey(1:jmax) = zya(1:jmax) ; zey(jmax+1) = zya(jmax) + zdy(jmax)
 zez(1:kmax) = zza(1:kmax) ; zez(kmax+1) = zza(kmax) + zdz(kmax)
 dbdims = (/imax+1,jmax+1,kmax+1/)

 nprefix= len(trim(prefix))
 write(tmp1,"(i4)") nfile + 1000
 filename = 'output/' // trim(prefix) // tmp1 // '.silo'

 error = dbcreate(trim(filename),len(trim(filename)),DB_CLOBBER,DB_LOCAL,trim(prefix),nprefix,DB_SILO_BACKEND,DBFile)

 error = dbmkoptlist(5, optionListID)
 !error = dbaddcopt(optionListID, DBOPT_COORDSYS, DB_CARTESIAN)
 error = dbaddcopt(optionListID, DBOPT_XLABEL, 'X_1', 3)
 error = dbaddcopt(optionListID, DBOPT_YLABEL, 'X_2', 3)
 error = dbaddcopt(optionListID, DBOPT_ZLABEL, 'X_3', 3)
 error = dbaddiopt(optionListID, DBOPT_CYCLE, ncycle)
 error = dbaddropt(optionListID, DBOPT_TIME, time)

 !--- Each processor writes its own domain file (mesh + variables):
 error = dbputqm(DBFile, trim(prefix), nprefix, 'crd_1', 5, 'crd_2', 5, 'crd_3', 5, &
                 zex, zey, zez, dbdims, ndim, DB_FLOAT, DB_COLLINEAR, optionListID, error)

 error = dbfreeoptlist(optionListID)

 dbdims = (/imax,jmax,kmax/)

 error = dbputqv1(DBFile, 'density',     7, trim(prefix), nprefix, zro, dbdims, ndim, DB_F77NULL, 0, &
                 DB_FLOAT, DB_ZONECENT, DB_F77NULL, error)

 error = dbputqv1(DBFile, 'velocity_1', 10, trim(prefix), nprefix, zux, dbdims, ndim, DB_F77NULL, 0, &
                 DB_FLOAT, DB_ZONECENT, DB_F77NULL, error)

 error = dbputqv1(DBFile, 'velocity_2', 10, trim(prefix), nprefix, zuy, dbdims, ndim, DB_F77NULL, 0, &
                 DB_FLOAT, DB_ZONECENT, DB_F77NULL, error)

 error = dbputqv1(DBFile, 'velocity_3', 10, trim(prefix), nprefix, zuz, dbdims, ndim, DB_F77NULL, 0, &
                 DB_FLOAT, DB_ZONECENT, DB_F77NULL, error)

 error = dbputqv1(DBFile, 'pressure',    8, trim(prefix), nprefix, zpr, dbdims, ndim, DB_F77NULL, 0, &
                 DB_FLOAT, DB_ZONECENT, DB_F77NULL, error)

 error = dbclose(DBFile)


endif

nfile = nfile + 1


 6000 format('Wrote time slice to ',a20,' at time =',1pe12.5,' (ncycle =', i7,')')
 6001 format('Wrote ',a16,' to disk at time =',1pe12.5,' (ncycle =', i7,')')

return
end

